﻿--[[
Title: myCalendarBroker
Author: Istaran | Medivh - EU
Version: 2.0.0
Release date: 2013-06-02T23:11:37Z
myCalendarBroker is licensed under GPLv3.
The sound file "question-dialog.mp3" is licenced under CCPL. All credits go to Nathaniel McCallum <npmccallum@canonical.com> form Canonical.
]]


-- ace addon creation and constants ---

local myCalendarBroker = LibStub("AceAddon-3.0"):NewAddon("myCalendarBroker", "AceEvent-3.0")
local MCB = myCalendarBroker

-- library initialization
local L = LibStub("AceLocale-3.0"):GetLocale("myCalendarBroker", true)
local LDB = LibStub:GetLibrary('LibDataBroker-1.1')

-- cache
local print, string, tonumber, tostring, format, wipe, table, type, pairs, ipairs  = print, string, tonumber, tostring, format, wipe, table, type, pairs, ipairs
local _

-- constants and tables
MCB.F = {}																									-- frame container
MCB.dayEvents = {}																							-- handles event of the current day
MCB.tomorrowEvents = {}																						-- handles event of the following day
MCB.countSorted = {}																						-- handles status
MCB.colorSorted = {}
MCB.nameSorted = {}
MCB.VERSION = GetAddOnMetadata("myCalendarBroker", "Version")
MCB.FLASH = 1
MCB.STILL = 1
MCB.ICON = "Interface\\Addons\\myCalendarBroker\\art\\calendar.tga"
MCB.TT_LINE = format("|T%s:0.5:75:0:0:8:8:0:8:0:8:255:255:255|t", "Interface\\Buttons\\WHITE8X8")
MCB.COLOR_Y = "|cffFFD200"
MCB.COLOR_G = "|cff20FF20"
MCB.COLOR_O = "|cffFF5E0D"
MCB.COLOR_W = "|cffFFFFFF"

-- default settings
MCB.defaults = {
	global = {
		lookWeeks = 1,
		holidays = true,
		mapButton = true,
		dateFormat = 1,
		brokerDisplay = 1,	-- 1 => week day + events, 2 => week day only, 3 => events only, 4 => week day only if 0 event
		popup = true,
		playsound = true,
	},
}

-- addon data
MCB.status = {
	[1] = {
		name = CALENDAR_STATUS_INVITED,
		count = 0,
		color = "|cffFFD200",
		order = 1,
	},
	[2] = {
		name = CALENDAR_STATUS_ACCEPTED,
		count = 0,
		color = "|cff20FF20",
		order = 2,
	},
	[3] = {
		name = CALENDAR_STATUS_DECLINED,
		count = 0,
		color = "|cffFF2020",
		order = 4,
	},
	[4] = {
		name = CALENDAR_STATUS_CONFIRMED,
		count = 0,
		color = "|cff1BCC1B",
		order = 5,
	},
	[5] = {
		name = CALENDAR_STATUS_OUT,
		count = 0,
		color = "|cffFF0000",
		order = 7,
	},
	[6] = {
		name = CALENDAR_STATUS_STANDBY,
		count = 0,
		color = "|cffFF5E0D",
		order = 6,
	},
	[7] = {
		name = CALENDAR_STATUS_SIGNEDUP,
		count = 0,
		color = "|cff20FF20",
		order = 22,
	},
	[8] = {
		name = CALENDAR_STATUS_NOT_SIGNEDUP,
		count = 0,
		color = "|cffFFD200",
		order = 21,
	},
	[9] = {
		name = CALENDAR_STATUS_TENTATIVE,
		count = 0,
		color = "|cffFF7F3F",
		order = 3,
	},
}

local pvpPath
do
	local faction = UnitFactionGroup("player")
	if faction == "Alliance" then
		pvpPath = "Interface\\Calendar\\UI-Calendar-Event-PVP02"
	elseif faction == "Horde" then
		pvpPath = "Interface\\Calendar\\UI-Calendar-Event-PVP01"
	else
		pvpPath = "Interface\\Calendar\\UI-Calendar-Event-PVP"
	end
end

local ttPath = {
	[1] = "Interface\\LFGFrame\\LFGIcon-",
	[2] = "Interface\\LFGFrame\\LFGIcon-",
	[3] = pvpPath,
	[4] = "Interface\\Calendar\\MeetingIcon",
	[5] = "Interface\\Calendar\\UI-Calendar-Event-Other",
	[6] = "Interface\\LFGFrame\\LFGIcon-",
	holiday = "Interface\\Calendar\\Holidays\\",
}

local privilegePath ={
	CREATOR = format("  |T%s:15:15:0:0:64:64:4:60:4:60|t", "Interface\\GroupFrame\\UI-Group-LeaderIcon"),
	MODERATOR = format("  |T%s:15:15:0:0:64:64:4:60:4:60|t", "Interface\\GroupFrame\\UI-Group-AssistantIcon"),
	[""] = "",
}

local holidaySuffix = {
	START = "Start",
	ONGOING = "Start",
	END = "End",
	[""] = "",
}

local holidaySuffixUser = {
	START = L["Start"],
	ONGOING = L["On going"],
	END = L["End"],
	[""] = L["Lasts one day"],
}

local weekDays = {
	[1] = L["Sunday"],
	[2] = L["Monday"],
	[3] = L["Tuesday"],
	[4] = L["Wednesday"],
	[5] = L["Thursday"],
	[6] = L["Friday"],
	[7] = L["Saturday"],
}

local monthNames = {
	[1] = L["January"],
	[2] = L["February"],
	[3] = L["March"],
	[4] = L["April"],
	[5] = L["May"],
	[6] = L["June"],
	[7] = L["July"],
	[8] = L["August"],
	[9] = L["September"],
	[10] = L["October"],
	[11] = L["November"],
	[11] = L["December"],
}

local daysAbr = {
	[1] = L["Sun."],
	[2] = L["Mon."],
	[3] = L["Tue."],
	[4] = L["Wed."],
	[5] = L["Thu."],
	[6] = L["Fri."],
	[7] = L["Sat."],
}

-- housekeeping functions ------------------

local function deepCopy(src)
	local res = {}
    for k, v in pairs(src) do
        if type(v) == "table" then
            v = deepCopy(v)
        end
        res[k] = v
    end
    return res
end

local function timeFormat(time)
	if type(time) == "number" and time < 10 then
			time = "0" .. tostring(time)
	end
	return time
end



-- addon loading ---------------------------

function MCB:OnInitialize()
	MCB.db = LibStub("AceDB-3.0"):New("mCBsettings", MCB.defaults)
end

function MCB:OnEnable()
	OpenCalendar()
	local _, month, _, year = CalendarGetDate()
	CalendarSetAbsMonth(month, year)
	
	MCB.obj = LDB:NewDataObject("myCalendarBroker", {
		type = "data source",
		text = L["updating..."],
		icon = MCB.ICON,
		OnClick = function(self, btn)
			MCB:OnClick(btn)
		end,
		OnTooltipShow = function(tooltip)
			MCB:TTShow(tooltip)
		end,
	})
	
	MCB.F.timer = CreateFrame("Frame", nil, UIParent)
	MCB.F.timer:SetScript("OnUpdate",  function(self, elapsed) MCB.OnUpdate(self, elapsed) end)
	MCB.F.timer:Hide()
	
	MCB:ShowCalendarButton()
	
	MCB:RegisterEvent("CALENDAR_UPDATE_EVENT_LIST", "EventUpdater")
	MCB:RegisterEvent("CALENDAR_UPDATE_PENDING_INVITES", "EventUpdater")
	
	MCB.F.popTimer = CreateFrame("Frame", nil, UIParent)
	MCB.F.popTimer:SetScript("OnUpdate",  function(self, elapsed) MCB.PopUpTimer(self, elapsed) end)
	MCB.F.popTimer:Hide()
	
	MCB:EventUpdater()
end



-- pop up window related functions ---------

function MCB:PopUpTimer(elapsed)
	self.TimeSinceLastUpdate = (self.TimeSinceLastUpdate or 0) + elapsed
	if self.TimeSinceLastUpdate > MCB.timer then
		self.TimeSinceLastUpdate = 0
		MCB.F.popTimer:Hide()
		if MCB.F.popup and MCB.F.popup:IsVisible() then
			MCB.F.popup.ag:Play()
		elseif not MCB.F.popup or not MCB.F.popup:IsVisible() then
			MCB:ShowPopUp()
		end
	end
end

function MCB:ShowPopUp()
	if not MCB.F.popup then
		MCB:CreatePopUp()
	else
		local children = {MCB.F.popup:GetChildren()}
		for _, child in ipairs(children) do
			if child ~= MCB.F.arrow then
				child:Hide()
				child:ClearAllPoints()
			end
		end
	end
	
	local offset = 3
	local maxWidth = 0
	local foundEvent
	
	-- pending invites
	if MCB.status[1].count > 0 then
		local event = MCB:PopUpText()
		event:SetPoint("TOPLEFT", MCB.F.popup, "TOPLEFT", 3, -offset)
		local plural = ""
		if MCB.status[1].count > 1 then plural = L["PLURAL_STRING"] end
		event.txt:SetText(format(MCB.COLOR_W .. L["You have %s pending invite%s!"], MCB.COLOR_Y .. MCB.status[1].count .. MCB.COLOR_W, plural))
		event:SetWidth(event.txt:GetStringWidth() + 6)
		event:SetHeight(event.txt:GetStringHeight() + 6)
		local width = event:GetWidth()
		if width > maxWidth then maxWidth = width end
		offset = offset + event:GetHeight() * 1.5
		foundEvent = true
	end
	
	-- events of the day
	local serverH, serverM = GetGameTime()
	for i = 1, #MCB.dayEvents do		
		if MCB.dayEvents[i].calendarType == "PLAYER" or string.find(MCB.dayEvents[i].calendarType, "GUILD") then
			if serverH < MCB.dayEvents[i].hour + 1 or (serverH == MCB.dayEvents[i].hour + 1 and serverM <= MCB.dayEvents[i].minute) then
				if MCB.dayEvents[i].inviteStatus ~= 3 or MCB.dayEvents[i].inviteStatus ~= 5 then -- do not show declined or out events
					local event = MCB:PopUpText()
					event:SetPoint("TOPLEFT", MCB.F.popup, "TOPLEFT", 3, -offset)
					event.txt:SetText(format(MCB.COLOR_W .. L["%s is scheduled today at %s:%s (%s)"], MCB.COLOR_Y .. MCB.dayEvents[i].title .. MCB.COLOR_W, MCB.COLOR_Y .. timeFormat(MCB.dayEvents[i].hour), timeFormat(MCB.dayEvents[i].minute) .. MCB.COLOR_W, MCB.status[MCB.dayEvents[i].inviteStatus].color .. MCB.status[MCB.dayEvents[i].inviteStatus].name .. MCB.COLOR_W))
					event:SetWidth(event.txt:GetStringWidth() + 6)
					event:SetHeight(event.txt:GetStringHeight() + 6)
					local width = event:GetWidth()
					if width > maxWidth then maxWidth = width end
					offset = offset + event:GetHeight()
					foundEvent = true
				end
			end
		end
	end
	
	if foundEvent then
		if maxWidth > 400 then maxWidth = 400 end
		MCB.F.popup:SetHeight(offset + 3)
		MCB.F.popup:SetWidth(maxWidth)
		
		MCB.timer = 8
		MCB.F.popTimer:Show()
		
		MCB.F.popup:Show()
		if MCB.db.global.playsound then
			PlaySoundFile("Interface\\Addons\\myCalendarBroker\\art\\dialog-question.mp3", "SFX")
		end
	end
end

function MCB:CheckBrokerDisplay()
	local supported = {"Bazooka", "ChocolateBar", "Fortress", "Titan"}
	local brokerFrameName = {"BazookaPlugin_myCalendarBroker", "ChocolatemyCalendarBroker", "LegoFortressmyCalendarBroker", "TitanPanelmyCalendarBrokerButton"}

	local displayFrame
	
	for i, v in ipairs(supported) do
		if IsAddOnLoaded(v) then
			displayFrame = brokerFrameName[i]
		end
	end
	
	return displayFrame
end

function MCB:CreatePopUp()
	local displayFrame = MCB:CheckBrokerDisplay()
	
	MCB.F.popup = CreateFrame("Frame", nil, UIParent)
	
	if displayFrame then		
		local sensorFrame = CreateFrame("Frame", nil, UIParent)
		sensorFrame:SetWidth(1)
		sensorFrame:SetHeight(1)
		sensorFrame:ClearAllPoints()
		sensorFrame:SetPoint("CENTER", displayFrame, "CENTER")
		sensorFrame:EnableMouse(false)
		sensorFrame:SetMovable(false)
		sensorFrame:Show()
		
		local x, y = sensorFrame:GetCenter()
		local UIWidth = UIParent:GetWidth()
		local UIHeight = UIParent:GetHeight()
		local displayPoint, popPoint, arrowPoint, rotateArrow
		local arrowX, ofy = 30, 20
		if x > UIWidth / 2 and y > UIHeight / 2 then
			displayPoint = "BOTTOMRIGHT"
			popPoint = "TOPRIGHT"
			arrowPoint = "BOTTOM"
			ofy = -ofy
			arrowX = -arrowX
		elseif x > UIWidth / 2 and y <= UIHeight / 2 then
			displayPoint = "TOPRIGHT"
			popPoint = "BOTTOMRIGHT"
			arrowPoint = "TOP"
			arrowX = -arrowX
			rotateArrow = true
		elseif x <= UIWidth / 2 and y > UIHeight / 2 then
			displayPoint = "BOTTOMLEFT"
			popPoint = "TOPLEFT"
			arrowPoint = "BOTTOM"
			ofy = -ofy
		else
			displayPoint = "TOPLEFT"
			popPoint = "BOTTOMLEFT"
			arrowPoint = "TOP"
			rotateArrow = true
		end
		
		sensorFrame:Hide()
		
		MCB.F.popup:ClearAllPoints()
		MCB.F.popup:SetParent(displayFrame)
		MCB.F.popup:SetPoint(popPoint, displayFrame, displayPoint, 0, ofy)
		
		MCB.F.arrow = CreateFrame("Frame", "mycalendarbroker_arrow_frame", MCB.F.popup)
		MCB.F.arrow:SetWidth(20)
		MCB.F.arrow:SetHeight(20)
		MCB.F.arrow:ClearAllPoints()
		MCB.F.arrow:SetPoint(arrowPoint, MCB.F.popup, popPoint, arrowX, 0)
		MCB.F.arrow.tex = MCB.F.arrow:CreateTexture(nil, "Background")
		MCB.F.arrow.tex:SetTexture("Interface\\Addons\\myCalendarBroker\\art\\arrow_pop.tga")
		MCB.F.arrow.tex:SetVertexColor(0, 0, 0, 0.8)
		MCB.F.arrow.tex:SetAllPoints(MCB.F.arrow)
		if rotateArrow then
			local angle = math.rad(180)
			local cos, sin = math.cos(angle), math.sin(angle)
			MCB.F.arrow.tex:SetTexCoord((sin - cos), -(cos + sin), -cos, -sin, sin, -cos, 0, 0)
		end
		
	else
		MCB.F.popup:ClearAllPoints()
		MCB.F.popup:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMRIGHT", -20, 20)		
	end
	
	MCB.F.popup:SetFrameStrata("High")
	
	MCB.F.popup.tex = MCB.F.popup:CreateTexture(nil, "Background")
	MCB.F.popup.tex:SetTexture(0, 0, 0, 0.8)
	MCB.F.popup.tex:SetAllPoints(MCB.F.popup)
	
	MCB.F.popup.ag = MCB.F.popup:CreateAnimationGroup()
	MCB.F.popup.ag:Stop()
	MCB.F.popup.fadeOut = MCB.F.popup.ag:CreateAnimation("Alpha")
	MCB.F.popup.fadeOut:SetChange(-1)
	MCB.F.popup.fadeOut:SetDuration(2)
	
	MCB.F.popup.ag:SetScript("OnFinished", function()  
		MCB.F.popup:Hide()
	end)
	
	MCB.F.popup:Hide()
end

function MCB:PopUpText()
	local textFrame = CreateFrame("Frame", nil, MCB.F.popup)
	textFrame.txt = textFrame:CreateFontString()
	textFrame.txt:SetShadowColor(0, 0, 0, 1)
	textFrame.txt:SetShadowOffset(1, -1)
	textFrame.txt:ClearAllPoints()
	textFrame.txt:SetAllPoints(textFrame)
	textFrame.txt:SetFont("Fonts\\FRIZQT__.TTF", 10)
	textFrame.txt:SetJustifyH("Left")
	
	return textFrame
end



-- mouse interaction -----------------------

function MCB:OnClick(btn)
	if btn == "LeftButton" then
		GameTimeFrame_OnClick(GameTimeFrame)
	elseif btn == "RightButton" then
		MCB:Options()
	end
end

function MCB:TTShow(tooltip)
	local texture = ""
	local isEventToday, isHoliday, isRaid
	
	-- title and date
	tooltip:AddLine(MCB.COLOR_G .. "myCalendarBroker " .. MCB.COLOR_O .. "v" .. MCB.VERSION)
	tooltip:AddLine(MCB.COLOR_W .. MCB:FormatedDate())
	tooltip:AddLine(" ")

	tooltip:AddLine(MCB.COLOR_W .. L["Status of next events:"])
	local isEvents = false
	for i = 1, #MCB.countSorted do
		if MCB.countSorted[i] ~= 0 then
			tooltip:AddDoubleLine(MCB.colorSorted[i] .. MCB.nameSorted[i] .. L[":"], MCB.colorSorted[i] .. MCB.countSorted[i])
			isEvents = true
		end
	end
	if not isEvents then
		tooltip:AddLine(L["No event"])
	end
	tooltip:AddDoubleLine(" ", MCB.TT_LINE)

	if MCB.dayEvents ~= nil then
		for i = 1, #MCB.dayEvents do
			if MCB.dayEvents[i].calendarType == "PLAYER" or string.find(MCB.dayEvents[i].calendarType, "GUILD") then
				isEventToday = true
			elseif MCB.dayEvents[i].calendarType == "HOLIDAY" then
				isHoliday = true
			elseif MCB.dayEvents[i].calendarType == "RAID_LOCKOUT" then
				isRaid = true
			end
		end
		
		-- events of the day
		if isEventToday then
			tooltip:AddLine(" ")
			tooltip:AddLine(MCB.COLOR_W .. L["Today events:"])
			for i = 1, #MCB.dayEvents do		
				if MCB.dayEvents[i].calendarType == "PLAYER" or string.find(MCB.dayEvents[i].calendarType, "GUILD") then
					if MCB.dayEvents[i].eventType == 1 or MCB.dayEvents[i].eventType == 2 or MCB.dayEvents[i].eventType == 6 then
						texture = format("|T%s:15:15:0:0:64:64:4:60:4:60|t", ttPath[MCB.dayEvents[i].eventType] .. MCB.dayEvents[i].texture)
					else
						texture = format("|T%s:15:15:0:0:64:64:4:60:4:60|t", ttPath[MCB.dayEvents[i].eventType])
					end
					tooltip:AddDoubleLine(texture .. " " .. MCB.dayEvents[i].title, timeFormat(MCB.dayEvents[i].hour) .. ":" .. timeFormat(MCB.dayEvents[i].minute))
					tooltip:AddDoubleLine(" ", MCB.status[MCB.dayEvents[i].inviteStatus].color .. MCB.status[MCB.dayEvents[i].inviteStatus].name .. privilegePath[MCB.dayEvents[i].modStatus])
				end
			end
			tooltip:AddDoubleLine(" ", MCB.TT_LINE)
		end
		
		-- holydays of the day
		if isHoliday and MCB.db.global.holidays then
			tooltip:AddLine(" ")
			tooltip:AddLine(MCB.COLOR_W .. L["Today world events:"])
			for i = 1, #MCB.dayEvents do
				if MCB.dayEvents[i].calendarType == "HOLIDAY" then
					texture = format("|T%s:15:15:0:0:128:128:2:90:2:90|t", "Interface\\Calendar\\Holidays\\" .. MCB.dayEvents[i].texture .. holidaySuffix[MCB.dayEvents[i].sequenceType])
					tooltip:AddDoubleLine(texture .. " " .. MCB.dayEvents[i].title, timeFormat(MCB.dayEvents[i].hour) .. ":" .. timeFormat(MCB.dayEvents[i].minute))
					tooltip:AddDoubleLine(" ", MCB.COLOR_G .. holidaySuffixUser[MCB.dayEvents[i].sequenceType])
				end
			end
			tooltip:AddDoubleLine(" ", MCB.TT_LINE)
		end
	end
	
	-- events of the next day
	if MCB.tomorrowEvents ~= nil then
		for i = 1, #MCB.tomorrowEvents do			
			if MCB.tomorrowEvents[i].calendarType == "PLAYER" or string.find(MCB.tomorrowEvents[i].calendarType, "GUILD") then
				tooltip:AddLine(" ")
				tooltip:AddLine(MCB.COLOR_W .. L["Tomorrow events:"])
				
				if MCB.tomorrowEvents[i].eventType == 1 or MCB.tomorrowEvents[i].eventType == 2 or MCB.tomorrowEvents[i].eventType == 6 then
					texture = format("|T%s:15:15:0:0:64:64:4:60:4:60|t", ttPath[MCB.tomorrowEvents[i].eventType] .. MCB.tomorrowEvents[i].texture)
				else
					texture = format("|T%s:15:15:0:0:64:64:4:60:4:60|t", ttPath[MCB.tomorrowEvents[i].eventType])
				end
				tooltip:AddDoubleLine(texture .. " " .. MCB.tomorrowEvents[i].title, timeFormat(MCB.tomorrowEvents[i].hour) .. ":" .. timeFormat(MCB.tomorrowEvents[i].minute))
				tooltip:AddDoubleLine(" ", MCB.status[MCB.tomorrowEvents[i].inviteStatus].color .. MCB.status[MCB.tomorrowEvents[i].inviteStatus].name .. privilegePath[MCB.tomorrowEvents[i].modStatus])
				tooltip:AddDoubleLine(" ", MCB.TT_LINE)
			end
		end
	end
	
	-- hints
	tooltip:AddLine(" ")
	tooltip:AddDoubleLine(MCB.COLOR_G .. L["Hints:"], " ")
	tooltip:AddDoubleLine(format("%s to toggle game calendar", MCB.COLOR_O .. L["Left-clic"] .. MCB.COLOR_G))
	tooltip:AddDoubleLine(format("%s to configure", MCB.COLOR_O .. L["Right-clic"] .. MCB.COLOR_G))
end

function MCB:FormatedDate()
	local curDayName, curMonth, curDay, curYear = CalendarGetDate()
	if MCB.db.global.dateFormat == 1 then	-- europe
		return weekDays[curDayName] .. ", " .. curDay .. " " .. monthNames[curMonth] .. " " .. curYear
	elseif MCB.db.global.dateFormat == 2 then	-- US
		return weekDays[curDayName] .. ", " .. monthNames[curMonth] .. " " .. curDay .. ", " .. curYear
	elseif MCB.db.global.dateFormat == 3 then	-- big endian
		return curYear .. " " .. monthNames[curMonth] .. " " .. curDay .. ", " .. weekDays[curDayName]
	end
end



-- events ----------------------------------

function MCB:EventUpdater()
	local dayToScan = 7
	if MCB.db.global.lookWeeks == 2 then
		dayToScan = 14
	elseif MCB.db.global.lookWeeks == 3 then
		dayToScan = 28
	end
		
	local _, curMonth, curDay = CalendarGetDate()
	local month, _, numDayInMonth = CalendarGetMonth(0)
	
	if month == curMonth then
		local monthOffset = 0
		for i in ipairs(MCB.status) do
			MCB.status[i].count = 0
		end
		
		local dayEvents = {}
		if #MCB.dayEvents > 0 then
			dayEvents = deepCopy(MCB.dayEvents)
		end
		local numInvites = MCB.status[1].count
		
		wipe(MCB.dayEvents)
		wipe(MCB.tomorrowEvents)
		wipe(MCB.countSorted)
		wipe(MCB.colorSorted)
		wipe(MCB.nameSorted)
	
		-- month/year jump
		for day = curDay, curDay + dayToScan do
			if day > numDayInMonth then
				day = day - numDayInMonth
				monthOffset = 1
			end
			local numEventToday = CalendarGetNumDayEvents(monthOffset, day)
			if numEventToday ~= 0 then
				for eventIndex = 1, numEventToday do
					local title, hour, minute, calendarType, sequenceType, eventType, texture, modStatus, inviteStatus = CalendarGetDayEvent(monthOffset, day, eventIndex)
					if calendarType == "PLAYER" or calendarType == "GUILD_EVENT" then
						MCB.status[inviteStatus].count = MCB.status[inviteStatus].count + 1
					end
					if day == curDay then
						MCB.dayEvents[eventIndex] = {
							title = title or "",
							hour = hour or "",
							minute = minute or "",
							calendarType = calendarType or "",
							sequenceType = sequenceType or "",
							eventType = eventType or "",
							texture = texture or "",
							modStatus = modStatus or "",
							inviteStatus = inviteStatus or "",
						}
					end
					if day == curDay + 1 then
						MCB.tomorrowEvents[eventIndex] = {
							title = title or "",
							hour = hour or "",
							minute = minute or "",
							calendarType = calendarType or "",
							sequenceType = sequenceType or "",
							eventType = eventType or "",
							texture = texture or "",
							modStatus = modStatus or "",
							inviteStatus = inviteStatus or "",
						}
					end
				end
			end
		end
	
		MCB.status[1].count = MCB.status[1].count + MCB.status[8].count
		MCB.status[8].count = 0
		MCB.status[2].count = MCB.status[2].count + MCB.status[7].count
		MCB.status[7].count = 0
	
		for i, v in ipairs(MCB.status) do
			MCB.countSorted[MCB.status[i].order] = MCB.status[i].count
			MCB.colorSorted[MCB.status[i].order] = MCB.status[i].color
			MCB.nameSorted[MCB.status[i].order] = MCB.status[i].name
		end
		
		MCB.obj.text = MCB:FillBrokerObj()		
		MCB.brokerText = nil
		MCB.brokerTextInvit = ""
		for i, v in ipairs(MCB.countSorted) do
			if MCB.countSorted[i] ~= 0 and not MCB.brokerText then
				MCB.brokerText = MCB.countSorted[i]
				if MCB.status[1].count > 0 then
					for j = 0, string.len(MCB.brokerText) do
						MCB.brokerTextInvit = MCB.brokerTextInvit .. " "
					end
				end
				MCB.brokerText = MCB.colorSorted[i] .. MCB.brokerText .. "|r"
			elseif MCB.countSorted[i] ~= 0 then
				MCB.brokerText = MCB.brokerText .. "/" .. MCB.colorSorted[i] .. MCB.countSorted[i] .. "|r"
				if MCB.brokerTextInvit then
					MCB.brokerTextInvit = MCB.brokerTextInvit .. "/" .. MCB.colorSorted[i] .. MCB.countSorted[i] .. "|r"
				end
			end
		end
		
		MCB.brokerText = MCB.brokerText or 0		
		MCB.obj.text = MCB:FillBrokerObj(MCB.brokerText)
		
		if MCB.status[1].count > 0 then
			MCB.F.timer:Show()
		elseif MCB.F.timer:IsVisible() then
			MCB.F.timer:Hide()
			MCB.obj.icon = MCB.ICON
			MCB.obj.text = MCB:FillBrokerObj(MCB.brokerText)
		end
		
		
		local eventsModified
		if #MCB.dayEvents ~= #dayEvents then
			eventsModified = true
		else
			for i in ipairs(MCB.dayEvents) do
				for k in pairs(MCB.dayEvents[i]) do
					if MCB.dayEvents[i][k] ~= dayEvents[i][k] then
						eventsModified = true
						break
					end
				end
				if eventsModified then break end
			end
		end
		
		if (eventsModified or MCB.status[1].count ~= numInvites) and not MCB.F.popTimer:IsVisible() and MCB.db.global.popup then
			MCB.timer = 20
			MCB.F.popTimer:Show()
		end
	end
end

function MCB:FillBrokerObj(brokerText)
	local text
	local curDayName, _, curDay = CalendarGetDate()
	brokerText = brokerText or ""
	
	if MCB.db.global.brokerDisplay == 1 then
		text = format(L["%s %s | %s ev."], daysAbr[curDayName], curDay, brokerText)
	elseif MCB.db.global.brokerDisplay == 2 then
		text = daysAbr[curDayName] .. " " .. curDay
	elseif MCB.db.global.brokerDisplay == 3 then
		text = format(L["%s ev."], brokerText)
	else
		local numE = 0
		for i in ipairs(MCB.status) do
			numE = numE + MCB.status[i].count
		end
		if numE > 0 then
			text = format(L["%s %s | %s ev."], daysAbr[curDayName], curDay, brokerText)
		else
			text = daysAbr[curDayName] .. " " .. curDay
		end
	end
	
	return text
end



-- on update -------------------------------

function MCB:OnUpdate(elapsed)
	self.TimeSinceLastUpdate = (self.TimeSinceLastUpdate or 0) + elapsed
	if self.TimeSinceLastUpdate > MCB.STILL + MCB.FLASH then
		MCB.obj.text = MCB:FillBrokerObj(MCB.brokerText)
		MCB.obj.icon = MCB.ICON
		self.TimeSinceLastUpdate = 0
	elseif self.TimeSinceLastUpdate > MCB.STILL then
		MCB.obj.text = MCB:FillBrokerObj(MCB.brokerTextInvit)
		MCB.obj.icon = ""
	end
end



-- config ----------------------------------

function MCB:ShowCalendarButton()
	if not MCB.db.global.mapButton and GameTimeFrame:IsVisible() then
		GameTimeFrame:Hide()
	elseif MCB.db.global.mapButton and not GameTimeFrame:IsVisible() then
		GameTimeFrame:Show()
	end
end

function MCB:Options()
	if not MCB.F.opt then
		MCB.F.opt = CreateFrame("Frame")
	end
	
	MCB.F.opt.displayMode = "MENU"
	local info = {}
	MCB.F.opt.initialize = function(self, level)
	    if not level then
			return
		end
		if level == 1 then
			wipe(info)
			info.isTitle = true
	        info.text = L["myCalendarBroker options"]
	        info.notCheckable = true
	        UIDropDownMenu_AddButton(info, level)
			
			wipe(info)
			info.text = L["Broker display"]
			info.hasArrow = true
			info.value = "brokerdisplay"
	        info.notCheckable = true
	        UIDropDownMenu_AddButton(info, level)
			
			wipe(info)
			info.text = L["Pop up"]
			info.hasArrow = true
			info.value = "popup"
	        info.notCheckable = true
	        UIDropDownMenu_AddButton(info, level)
			
			wipe(info)
			info.text = L["Play a sound on pop up"]
			info.hasArrow = true
			info.value = "playsound"
	        info.notCheckable = true
	        UIDropDownMenu_AddButton(info, level)
			
			wipe(info)
			info.text = L["Show calendar map button"]
			info.hasArrow = true
			info.value = "minimap"
	        info.notCheckable = true
	        UIDropDownMenu_AddButton(info, level)
			
			wipe(info)
			info.isTitle = true
	        info.notCheckable = true
			info.text = ""
	        UIDropDownMenu_AddButton(info, level)
			
			wipe(info)
			info.text = L["Monitoring"]
			info.hasArrow = true
			info.value = "monitoring"
	        info.notCheckable = true
	        UIDropDownMenu_AddButton(info, level)
			
			wipe(info)
			info.text = L["Show world events"]
			info.hasArrow = true
			info.value = "holiday"
	        info.notCheckable = true
	        UIDropDownMenu_AddButton(info, level)
			
			wipe(info)
			info.text = L["Date format"]
			info.hasArrow = true
			info.value = "dateformat"
	        info.notCheckable = true
	        UIDropDownMenu_AddButton(info, level)
			
			wipe(info)
			info.isTitle = true
	        info.notCheckable = true
			info.text = ""
	        UIDropDownMenu_AddButton(info, level)
			
			wipe(info)
			info.text = L["Close menu"]
			info.func = function() CloseDropDownMenus() end
	        info.notCheckable = true
	        UIDropDownMenu_AddButton(info, level)
		
		elseif level == 2 then
			if UIDROPDOWNMENU_MENU_VALUE == "monitoring" then
				wipe(info)
				info.text = L["One week"]
				info.checked = MCB.db.global.lookWeeks == 1
				info.func = function()
					MCB.db.global.lookWeeks = 1
					MCB:EventUpdater()
				end
				UIDropDownMenu_AddButton(info, level)
			
				info.text = L["Two weeks"]
				info.checked = MCB.db.global.lookWeeks == 2
				info.func = function()
					MCB.db.global.lookWeeks = 2
					MCB:EventUpdater()
				end
				UIDropDownMenu_AddButton(info, level)
				
				info.text = L["One month"]
				info.checked = MCB.db.global.lookWeeks == 3
				info.func = function()
					MCB.db.global.lookWeeks = 3
					MCB:EventUpdater()
				end
				UIDropDownMenu_AddButton(info, level)
				
			elseif UIDROPDOWNMENU_MENU_VALUE == "brokerdisplay" then
				wipe(info)
				info.text = L["Week days and events"]
				info.checked = MCB.db.global.brokerDisplay == 1
				info.func = function()
					MCB.db.global.brokerDisplay = 1
					if not MCB.F.timer:IsVisible() then
						MCB.obj.text = MCB:FillBrokerObj(MCB.brokerText)
					end
				end
				UIDropDownMenu_AddButton(info, level)
			
				info.text = L["Week days only"]
				info.checked = MCB.db.global.brokerDisplay == 2
				info.func = function()
					MCB.db.global.brokerDisplay = 2
					if not MCB.F.timer:IsVisible() then
						MCB.obj.text = MCB:FillBrokerObj(MCB.brokerText)
					end
				end
				UIDropDownMenu_AddButton(info, level)
				
				info.text = L["Events only"]
				info.checked = MCB.db.global.brokerDisplay == 3
				info.func = function()
					MCB.db.global.brokerDisplay = 3
					if not MCB.F.timer:IsVisible() then
						MCB.obj.text = MCB:FillBrokerObj(MCB.brokerText)
					end
				end
				UIDropDownMenu_AddButton(info, level)
				
				info.text = L["Week days only if no event"]
				info.checked = MCB.db.global.brokerDisplay == 4
				info.func = function()
					MCB.db.global.brokerDisplay = 4
					if not MCB.F.timer:IsVisible() then
						MCB.obj.text = MCB:FillBrokerObj(MCB.brokerText)
					end
				end
				UIDropDownMenu_AddButton(info, level)
				
			elseif UIDROPDOWNMENU_MENU_VALUE == "popup" then
				wipe(info)
				info.text = L["Enable"]
				info.checked = MCB.db.global.popup
				info.func = function()
					MCB.db.global.popup = true
				end
				UIDropDownMenu_AddButton(info, level)
			
				info.text = L["Disable"]
				info.checked = not MCB.db.global.popup
				info.func = function()
					MCB.db.global.popup = false
				end
				UIDropDownMenu_AddButton(info, level)
				
			elseif UIDROPDOWNMENU_MENU_VALUE == "playsound" then
				wipe(info)				
				info.text = L["Enable"]
				info.checked = MCB.db.global.playsound
				info.func = function()
					MCB.db.global.playsound = true
				end
				UIDropDownMenu_AddButton(info, level)
				
				info.text = L["Disable"]
				info.checked = not MCB.db.global.playsound
				info.func = function()
					MCB.db.global.playsound = false
				end
				UIDropDownMenu_AddButton(info, level)
				
			elseif UIDROPDOWNMENU_MENU_VALUE == "holiday" then
				wipe(info)
				info.text = L["Yes"]
				info.checked = MCB.db.global.holidays
				info.func = function()
					MCB.db.global.holidays = true
				end
				UIDropDownMenu_AddButton(info, level)
			
				info.text = L["No"]
				info.checked = not MCB.db.global.holidays
				info.func = function()
					MCB.db.global.holidays = false
				end
				UIDropDownMenu_AddButton(info, level)
			
			elseif UIDROPDOWNMENU_MENU_VALUE == "minimap" then
				wipe(info)
				info.text = L["Yes"]
				info.checked = MCB.db.global.mapButton
				info.func = function()
					MCB.db.global.mapButton = true
					MCB:ShowCalendarButton()
				end
				UIDropDownMenu_AddButton(info, level)
			
				info.text = L["No"]
				info.checked = not MCB.db.global.mapButton
				info.func = function()
					MCB.db.global.mapButton = false
					MCB:ShowCalendarButton()
				end
				UIDropDownMenu_AddButton(info, level)
			
			elseif UIDROPDOWNMENU_MENU_VALUE == "dateformat" then
				wipe(info)
				info.text = L["Day name, Day Month Year"]
				info.checked = MCB.db.global.dateFormat == 1
				info.func = function()
					MCB.db.global.dateFormat = 1
				end
				UIDropDownMenu_AddButton(info, level)
				
				info.text = L["Day name, Month Day, Year"]
				info.checked = MCB.db.global.dateFormat == 2
				info.func = function()
					MCB.db.global.dateFormat = 2
				end
				UIDropDownMenu_AddButton(info, level)
			
				info.text = L["Year Month Day, Day name"]
				info.checked = MCB.db.global.dateFormat == 3
				info.func = function()
					MCB.db.global.dateFormat = 3
				end
				UIDropDownMenu_AddButton(info, level)
			end
		end
	end

	local xcur, ycur = GetCursorPosition(UIParent)
	local UIScale = UIParent:GetScale()
	ToggleDropDownMenu(1, nil, MCB.F.opt, UIParent, xcur / UIScale, ycur / UIScale)
end
